@@ -1,6 +1,6 @@
 /* Python interface to breakpoints
 
-   Copyright (C) 2008-2025 Free Software Foundation, Inc.
+   Copyright (C) 2008-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -35,8 +35,7 @@
 #include "linespec.h"
 #include "gdbsupport/common-utils.h"
 
-extern PyTypeObject breakpoint_location_object_type
-    CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("breakpoint_location_object");
+extern PyTypeObject breakpoint_location_object_type;
 
 struct gdbpy_breakpoint_location_object
 {
@@ -1353,8 +1352,8 @@ gdbpy_breakpoint_modified (struct breakpoint *b)
 
 
 /* Initialize the Python breakpoint code.  */
-static int CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
-gdbpy_initialize_breakpoints (void)
+static int
+gdbpy_initialize_breakpoints ()
 {
   int i;
 
@@ -1389,7 +1388,7 @@ gdbpy_initialize_breakpoints (void)
 
 /* Initialize the Python BreakpointLocation code.  */
 
-static int CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
+static int
 gdbpy_initialize_breakpoint_locations ()
 {
   return gdbpy_type_ready (&breakpoint_location_object_type);
@@ -1645,7 +1644,7 @@ bploc_filepath (struct symtab *bploc_symtab)
     {
     }
 
-  return host_string_to_python_string (bploc_symtab->filename);
+  return host_string_to_python_string (bploc_symtab->filename ());
 }
 
 /* Python function to get the source file name path and line number
@@ -1764,7 +1763,7 @@ bplocpy_repr (PyObject *py_self)
 			  paddress (self->bp_loc->owner->gdbarch,
 				    self->bp_loc->requested_address));
   if (self->bp_loc->symtab != nullptr)
-    str += string_printf (" source=%s:%d", self->bp_loc->symtab->filename,
+    str += string_printf (" source=%s:%d", self->bp_loc->symtab->filename (),
 			  self->bp_loc->line_number);
 
   const auto fn_name = self->bp_loc->function_name.get ();
