@@ -1,5 +1,5 @@
 /* Register protocol definition structures for the GNU Debugger
-   Copyright (C) 2001-2022 Free Software Foundation, Inc.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef REGFORMATS_REGDEF_H
-#define REGFORMATS_REGDEF_H
+#ifndef GDB_REGFORMATS_REGDEF_H
+#define GDB_REGFORMATS_REGDEF_H
 
 namespace gdb {
 
@@ -64,4 +64,4 @@ struct reg
 
 } /* namespace gdb */
 
-#endif /* REGFORMATS_REGDEF_H */
+#endif /* GDB_REGFORMATS_REGDEF_H */
